@@ -465,9 +465,6 @@ void _acb_poly_root_inclusion(acb_t r, const acb_t m,
 slong _acb_poly_validate_roots(acb_ptr roots,
         acb_srcptr poly, slong len, slong prec);
 
-void _acb_poly_refine_roots_durand_kerner(acb_ptr roots,
-        acb_srcptr poly, slong len, slong prec);
-
 slong _acb_poly_find_roots(acb_ptr roots,
     acb_srcptr poly,
     acb_srcptr initial, slong len, slong maxiter, slong prec);
@@ -567,6 +564,8 @@ void acb_poly_tan_series(acb_poly_t g, const acb_poly_t h, slong n, slong prec);
 
 void _acb_poly_sinc_series(acb_ptr g, acb_srcptr h, slong hlen, slong n, slong prec);
 void acb_poly_sinc_series(acb_poly_t g, const acb_poly_t h, slong n, slong prec);
+void _acb_poly_sinc_pi_series(acb_ptr g, acb_srcptr h, slong hlen, slong n, slong prec);
+void acb_poly_sinc_pi_series(acb_poly_t g, const acb_poly_t h, slong n, slong prec);
 
 void _acb_poly_lambertw_series(acb_ptr res, acb_srcptr z, slong zlen, const fmpz_t k, int flags, slong len, slong prec);
 void acb_poly_lambertw_series(acb_poly_t res, const acb_poly_t z, const fmpz_t k, int flags, slong len, slong prec);
