@@ -5,7 +5,7 @@
 
     FLINT is free software: you can redistribute it and/or modify it under
     the terms of the GNU Lesser General Public License (LGPL) as published
-    by the Free Software Foundation; either version 2.1 of the License, or
+    by the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.  See <https://www.gnu.org/licenses/>.
 */
 
@@ -16,8 +16,8 @@
 #define FQ_ZECH_EMBED_INLINE
 #define FQ_EMBED_TEMPLATES_INLINE
 #else
-#define FQ_ZECH_EMBED_INLINE static __inline__
-#define FQ_EMBED_TEMPLATES_INLINE static __inline__
+#define FQ_ZECH_EMBED_INLINE static inline
+#define FQ_EMBED_TEMPLATES_INLINE static inline
 #endif
 
 #include "fq_zech_types.h"
